#since object members in this class are created dynamically, disable pylint error with:
#pylint: disable=no-member
class MarkdownDataObjectReplicaSet:

	''' Markdown representation of DataObjectReplicaSet class.'''

	def convert_to_markdown(self):

		''' Get markdown representation.'''

		output = ''

		return output
